#include <queue>
#include <algorithm> // for max_element
#include <string>    // for strcmp, strcpy
#include <cstring>   // for memset
#include <math.h>    // for floor and pow
using namespace std;

#define IS_REAL_2D(P) (!mxIsComplex(P) && mxGetNumberOfDimensions(P) == 2)

// structure of a rectangle set
struct Rset {
  short lo[4]; // B1_lo, E1_lo, B2_lo, E2_lo
  short hi[4]; // B1_hi, E1_hi, B2_hi, E2_hi
  bool  bValid;
  
  void disp() {
    mexPrintf("lo=[%d,%d,%d,%d], hi=[%d,%d,%d,%d], bValid=%d\n", \
            lo[0],lo[1],lo[2],lo[3], hi[0],hi[1],hi[2],hi[3], bValid);
  }

  void setValid(bool vv) {
    bValid = vv;
  }

  bool isValid() {
    return bValid;
  }

};
/* ==================structure of a cubic set============
                      add by Jiabei, 2015-3-30          */
struct Cset{
    short lo[6];
    short hi[6];
    bool bValid;
    
    void disp() {
        mexPrintf("lo=[%d,%d,%d,%d,%d,%d], hi=[%d,%d,%d,%d,%d,%d], bValid=%d\n", \
                  lo[0],lo[1],lo[2],lo[3], lo[4], lo[5], hi[0],hi[1],hi[2],hi[3],hi[4],hi[5], bValid);
    }
    
    void setValid(bool vv) {
        bValid = vv;
    }
    
    bool isValid() {
        return bValid;
    }
    
};
struct StateTri {
    float lbound;
    float ubound;
    Cset R;
    
    void disp() {
        //mexPrintf("lb=%e, ub=%e lo=[%d,%d,%d,%d], hi=[%d,%d,%d,%d]\n", lbound, ubound,\
        //        R.lo[0],R.lo[1],R.lo[2],R.lo[3], R.hi[0],R.hi[1],R.hi[2],R.hi[3]);
        mexPrintf("lb=%e\t ub=%e\t B1=[%d,%d]\t E1=[%d,%d]\t B2=[%d,%d]\t E2=[%d,%d]\t B3=[%d,%d]\t E3=[%d,%d]\n", \
                  lbound, ubound, R.lo[0],R.hi[0],R.lo[1],R.hi[1], \
                                  R.lo[2],R.hi[2],R.lo[3],R.hi[3], \
                                  R.lo[4],R.hi[4],R.lo[5],R.hi[5]);
    }
};
// Comparator of the priority queue
class CompareStateTri{
public:
    bool operator () (const StateTri& R1, const StateTri& R2) {
        if (R1.lbound > R2.lbound) return true;
        return false;
    }
};

/*======================================*/

struct State {
  float lbound;
  float ubound;
  Rset R;
  
  void disp() {
    //mexPrintf("lb=%e, ub=%e lo=[%d,%d,%d,%d], hi=[%d,%d,%d,%d]\n", lbound, ubound,\
    //        R.lo[0],R.lo[1],R.lo[2],R.lo[3], R.hi[0],R.hi[1],R.hi[2],R.hi[3]);
    mexPrintf("lb=%e\t ub=%e\t B1=[%d,%d]\t E1=[%d,%d]\t B2=[%d,%d]\t E2=[%d,%d]\n", lbound, ubound,\
            R.lo[0],R.hi[0],R.lo[1],R.hi[1], R.lo[2],R.hi[2],R.lo[3],R.hi[3]);
  }
};

// Comparator of the priority queue
class CompareState {
public:
  bool operator () (const State& R1, const State& R2) {
    if (R1.lbound > R2.lbound) return true;
    return false;
  }
};

// Utility functions
void  printUsg();
void  split(const Rset& R, Rset& R1, Rset& R2);
void  split(const Cset& R, Cset& R1, Cset& R2);
void  split(const Rset& R, Rset& R1, Rset& R2, short L, short T, short m, short n);
void  rectify(Rset& R);
void  rectify(Cset& R);
void  copyR(Rset& Rin, Rset& Rout);
void  copyR(Cset& Rin, Cset& Rout);
void  copyS(State& Sin, State& Sout);
void  copyS(StateTri& Sin, StateTri& Sout);
void  updateR(Rset& rect, Rset& R);
void  updateR(Cset& rect, Cset& R);
int   getBounds(float* S1r, float* S2r, const short& B, const Rset& R, float& lb, float& ub, const string dist);
int   getBounds(float* S1r, float* S2r, float* S3r, const short& B, const Cset& R, float& lb, float& ub, const string dist);
int   maxside(const Rset& R);
int   maxside(const Cset& R);
int   getParams(const int nrhs, const mxArray *prhs[], short& L, unsigned int& maxIter, short& nRes, string& dist, bool& isVerbose);
bool  isvalid(const Rset& R, const short L);
bool  isvalid(const Cset& R, const short L);
bool  isUniqR(const Rset& R);
bool  isUniqR(const Cset& R);
bool  isOverlap(Rset& rect, Rset& R);
bool  isOverlap(Cset& rect, Cset& R);
void  vSub(const float* v1, const float* v2, const short& B, float** out);
float vSum(const float* v, const short& B);
float vSqSum(const float* v, const short& B);
template<typename T> void setField(mxArray** A, const int structIdx, const char* fieldname, const int size, mxClassID mxType, const T* value);
template<typename T> bool ismember(T a, T interval_begin, T interval_end);
void check_conflict(Rset& R,  short *inValidSegs1, short n_invalid_1, short *inValidSegs2,short n_invalid_2);
void check_conflict(Cset& R, short *inValidSegs1, short n_invalid_1, short *inValidSegs2,short n_invalid_2, short *inValidSegs3, short n_invalid_3);
//void update_order(short *s, short n, short rm_value, short bidx );
//void sort_vector(float *s, short* idx, short n, int t);
short min2(short a, short b );
short max2(short a, short b );
void getValidSegment(short nSeg, short len, short *inValidSegs, short *validSegs);
bool check_seq_set(short * lo, short *hi, short *inValidSegs, short &n_invalid);

